Package: argparser
Type: Package
Title: Command-Line Argument Parser
Version: 0.7.2
Author: David J. H. Shih
Maintainer: David J. H. Shih <djh.shih@gmail.com>
Description: Cross-platform command-line argument parser written purely in R
    with no external dependencies. It is useful with the Rscript
    front-end and facilitates turning an R script into an executable script.
URL: https://bitbucket.org/djhshih/argparser
BugReports: https://bitbucket.org/djhshih/argparser/issues
Depends: methods
Suggests: testthat (>= 3.0.0)
License: GPL (>= 3)
RoxygenNote: 7.2.3
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2024-04-03 15:42:34 UTC; davids
Repository: CRAN
Date/Publication: 2024-04-04 05:03:14 UTC
Built: R 4.2.3; ; 2024-04-04 05:07:20 UTC; unix
