Package: argparser
Type: Package
Title: Command-Line Argument Parser
Version: 0.7.3
Authors@R: person(given = c("David", "J.", "H."),
                  family = "Shih",
                  role = c("aut", "cre"),
                  email = "djh.shih@gmail.com")
Description: Cross-platform command-line argument parser written purely in R
    with no external dependencies. It is useful with the Rscript
    front-end and facilitates turning an R script into an executable script.
URL: https://bitbucket.org/djhshih/argparser
BugReports: https://bitbucket.org/djhshih/argparser/issues
Depends: methods
Suggests: testthat (>= 3.0.0)
License: GPL (>= 3)
RoxygenNote: 7.2.3
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-02-24 04:51:00 UTC; davids
Author: David J. H. Shih [aut, cre]
Maintainer: David J. H. Shih <djh.shih@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-24 06:40:09 UTC
