% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/argparser.R
\name{spaces}
\alias{spaces}
\title{Space string.}
\usage{
spaces(n)
}
\arguments{
\item{n}{number of spaces}
}
\value{
a character string containing \code{n} spaces
}
\description{
Space string.
}
