% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/argparser.R
\name{print.arg.parser}
\alias{print.arg.parser}
\title{Print the help message for an arg.parser.}
\usage{
\method{print}{arg.parser}(x, ...)
}
\arguments{
\item{x}{an \code{arg.parser} object}

\item{...}{unused arguments}
}
\description{
This function prints the help message.
}
\details{
At the command line, we would use the \code{--help} or \code{-help} flag
to print the help message:
\code{$ script --help}
}
